/*
 *  TraceAlloc.h by Davide Libenzi (Trace allocator for iPhone/OSX)
 *  Copyright (C) 2008  Davide Libenzi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Davide Libenzi <davidel@xmailserver.org>
 *
 */

#ifndef _TRACEALLOC_H
#define _TRACEALLOC_H

#include <stdio.h>
#include <limits.h>
#include <errno.h>
#import <CoreFoundation/CoreFoundation.h>
#import <Foundation/Foundation.h>

#define TAF_BRIEF_SYMBOL (1 << 0)
#define TAF_FULL_NAME (1 << 1)

void TADumpBlocks(CFAllocatorRef hookATor, unsigned long mark, int fd, int flags);
void TADumpBlocksFile(CFAllocatorRef hookATor, unsigned long mark,
		      const char *path, int flags);
unsigned long TAAllocMark(CFAllocatorRef hookATor);
void TAAllocUnmark(CFAllocatorRef hookATor, int fd, int flags);
void TAAllocUnmarkFile(CFAllocatorRef hookATor, const char *path,
		       int flags);
CFAllocatorRef TAAllocatorCreate(CFAllocatorRef origATor);
void TATraceDefaultAllocator(void);

static inline void *TAMalloc(unsigned int size)
{
	return CFAllocatorAllocate(NULL, size, 0);
}

static inline void *TACalloc(unsigned int count, unsigned int size)
{
	if (size && count > (UINT_MAX / size)) {
		errno = EINVAL;
		return NULL;
	}
	return CFAllocatorAllocate(NULL, size * count, 0);
}

static inline void *TARealloc(void *ptr, unsigned int size)
{
	return CFAllocatorReallocate(NULL, ptr, size, 0);
}

static inline void TAFree(void *ptr)
{
	CFAllocatorDeallocate(NULL, ptr);
}

#endif

